SystemOrganization addCategory: #'Physics-Core'!
SystemOrganization addCategory: #'Physics-View'!
SystemOrganization addCategory: #'Physics-Tests'!

Object subclass: #PhObject
	instanceVariableNames: 'origin velocity mass acceleration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Physics-Core'!

!PhObject class methodsFor: 'as yet unclassified' stamp: 'srh 12/13/2008 00:25'!
at: aPoint
	^self basicNew initialize; origin: aPoint; yourself.! !

!PhObject class methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:38'!
at: aPoint mass: aMass
	^self basicNew initialize; origin: aPoint; mass: aMass; yourself.! !

!PhObject methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:14'!
acceleration
	^acceleration! !

!PhObject methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:40'!
addForce: aForce
	acceleration := acceleration + (aForce accelerationForMass: self mass)! !

!PhObject methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:35'!
applyPhysicsFor: duration
	"First apply the current velocity"
	origin := origin + (velocity * duration).
	"Now add the additional translation based on the net acceleration over the duration"
	origin := origin + (acceleration translationFor: duration).
	"Now update the velocity"
	velocity := velocity + (acceleration * duration).
	"Clear the accleration"
	self resetAcceleration.
! !

!PhObject methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 22:06'!
initialize
	self resetAcceleration.
	mass := 1.
	origin := 0@0.
	velocity := PhVector new.! !

!PhObject methodsFor: 'as yet unclassified' stamp: 'srh 12/13/2008 00:07'!
mass
	^mass! !

!PhObject methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:39'!
mass: kg
	mass := kg.! !

!PhObject methodsFor: 'as yet unclassified' stamp: 'srh 12/13/2008 00:26'!
origin: aPoint
	origin := aPoint.! !

!PhObject methodsFor: 'as yet unclassified' stamp: 'srh 12/12/2008 23:45'!
position
	^origin! !

!PhObject methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:16'!
resetAcceleration
	acceleration := PhAcceleration new! !

!PhObject methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:30'!
velocity: aVector
	velocity := aVector! !

Object subclass: #PhVector
	instanceVariableNames: 'point'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Physics-Core'!

PhVector subclass: #PhAcceleration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Physics-Core'!

!PhAcceleration methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:41'!
translationFor: seconds
	^PhVector magnitude: (self magnitude/2) * seconds squared direction: self direction.! !

PhVector subclass: #PhForce
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Physics-Core'!

!PhForce class methodsFor: 'as yet unclassified' stamp: 'srh 12/13/2008 00:10'!
new
	^self basicNew initialize;yourself! !

!PhForce methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:44'!
accelerationForMass: aMass
	" F = ma, therefore a = F/m "
	^PhVector magnitude: self magnitude / aMass direction: self direction.! !

!PhVector class methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:42'!
fromComponents: c
	^self new components: c;yourself
	! !

!PhVector class methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:41'!
magnitude: aMagnitude direction: aDirection
	^self new magnitude: aMagnitude direction: aDirection;yourself! !

!PhVector class methodsFor: 'as yet unclassified' stamp: 'srh 12/13/2008 00:12'!
new
	^self basicNew initialize; yourself.! !

!PhVector methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:37'!
* multiplier
	^self class magnitude: point r * multiplier direction: point degrees! !

!PhVector methodsFor: 'accessing' stamp: 'srh 12/13/2008 23:20'!
+ aVector
	^self class fromComponents: (self components + aVector components)! !

!PhVector methodsFor: 'accessing' stamp: 'srh 12/14/2008 20:37'!
= aVector
	^(aVector magnitude closeTo: self magnitude) 
		and: [(aVector direction closeTo: self direction)]! !

!PhVector methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:47'!
adaptToPoint: aPoint andSend: message
	^aPoint perform: message with: point! !

!PhVector methodsFor: 'accessing' stamp: 'srh 12/13/2008 00:02'!
angle
	^self direction! !

!PhVector methodsFor: 'accessing' stamp: 'srh 12/14/2008 20:36'!
components
	^point! !

!PhVector methodsFor: 'accessing' stamp: 'srh 12/14/2008 20:39'!
components: aPoint
	point := aPoint! !

!PhVector methodsFor: 'accessing' stamp: 'srh 12/14/2008 20:34'!
direction
	^ point degrees! !

!PhVector methodsFor: 'accessing' stamp: 'srh 12/14/2008 20:38'!
initialize
	super initialize.
	point := 0@0! !

!PhVector methodsFor: 'accessing' stamp: 'srh 12/14/2008 20:42'!
magnitude
	^ point r! !

!PhVector methodsFor: 'accessing' stamp: 'srh 12/14/2008 20:35'!
magnitude: magnitude direction: direction
	point := Point r: magnitude degrees: direction.! !

!PhVector methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:48'!
opposite
	^self class magnitude: self magnitude direction: (self direction + 180) // 360! !

!PhVector methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:43'!
printOn: aStream
	aStream nextPutAll: '->('.
	point r printOn: aStream.
	aStream nextPut: $,.
	point degrees printOn: aStream.
	aStream nextPut: $).
	! !

Object subclass: #PhWorld
	instanceVariableNames: 'size period objects gravity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Physics-Core'!

!PhWorld class methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:56'!
new
	^self basicNew initialize.! !

!PhWorld methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:55'!
addObject: anObject
	objects add: anObject.! !

!PhWorld methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 22:05'!
gravityFor: anObject
	^PhForce magnitude: ((anObject mass) * gravity) direction: 270.! !

!PhWorld methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 22:02'!
initialize
	objects := OrderedCollection new.
	period := 0.1.
	gravity := 9.81! !

!PhWorld methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 22:03'!
runFor: seconds.
	0 to: seconds by: period do: [ :i |
		Transcript show: i printString, ' seconds: '.
		objects do: [ :o |
			o addForce: (self gravityFor: o).
			o applyPhysicsFor: period.
			Transcript show: o position printString, ' '
		].
		Transcript cr.
	]! !

!Point methodsFor: '*physics' stamp: 'srh 12/14/2008 20:30'!
closeTo: aPoint
	^(x closeTo: aPoint x) and: [y closeTo: aPoint y].! !

TestCase subclass: #PhAccelerationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Physics-Tests'!

!PhAccelerationTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:23'!
testTranslationFor1s
	|a|
	a := PhAcceleration magnitude: 20 direction: 0.
	self assert: (PhVector magnitude: 10 direction: 0) equals: (a translationFor: 1).! !

!PhAccelerationTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:24'!
testTranslationFor2s
	|a|
	a := PhAcceleration magnitude: 20 direction: 0.
	self assert: (PhVector magnitude: 40 direction: 0) equals: (a translationFor: 2).! !

!PhAccelerationTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:25'!
testTranslationForHalfS
	|a|
	a := PhAcceleration magnitude: 20 direction: 0.
	self assert: (PhVector magnitude: 2.5 direction: 0) equals: (a translationFor: 0.5).! !

TestCase subclass: #PhForceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Physics-Tests'!

!PhForceTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:50'!
testAccelerationForMass
	self assert: (PhVector magnitude: 10 direction: 90) 
		equals: ((PhForce magnitude: 100 direction: 90) accelerationForMass: 10)! !

TestCase subclass: #PhObjectTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Physics-Tests'!

!PhObjectTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:37'!
testAddForce1
	|o|
	o := PhObject at: 0@0 mass: 5.
	o addForce: (PhForce magnitude: 10 direction: 45).
	self assert: (PhAcceleration magnitude: 2 direction: 45) equals: o acceleration.! !

!PhObjectTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:31'!
testMovement1s
	|o|
	o := PhObject at: 100@100.
	o velocity: (PhVector magnitude: 10 direction: 0).
	o applyPhysicsFor: 1.
	self assert: 110@100 equals: o position.! !

!PhObjectTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:34'!
testMovement2s
	|o|
	o := PhObject at: 100@100.
	o velocity: (PhVector magnitude: 10 direction: 0).
	o applyPhysicsFor: 2.
	self assert: 120@100 equals: o position.! !

!PhObjectTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 21:34'!
testMovementHalfS
	|o|
	o := PhObject at: 100@100.
	o velocity: (PhVector magnitude: 10 direction: 0).
	o applyPhysicsFor: 0.5.
	self assert: 105@100 equals: o position.! !

TestCase subclass: #PhVectorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Physics-Tests'!

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:27'!
assert: expected closeTo: actual
	self assert: (expected closeTo: actual)
		description:  (self comparingStringBetween: expected and: actual)! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 19:10'!
testAdd1
	| v1 v2 |
	v1 := PhVector magnitude: 1 direction: 180.
	v2 := PhVector magnitude: 1 direction: 180.
	self assert: (PhVector magnitude: 2 direction: 180) equals: (v1 + v2)! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/13/2008 23:08'!
testAdd2
	| v1 v2 |
	v1 := PhVector magnitude: 11 direction: 0.
	v2 := PhVector magnitude: 11 direction: 90.
	self assert: (PhVector magnitude: 15.55634918610404 direction: 45) equals: (v1 + v2)! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/13/2008 23:08'!
testAdd3
	| v1 v2 |
	v1 := PhVector magnitude: 10 direction: 180.
	v2 := PhVector magnitude: 5 direction: 270.
	self assert: (PhVector magnitude: 11.18033988749895 direction: 206.565051177078) equals: (v1 + v2)! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:21'!
testComponentsE
	self assert: 10@0 closeTo:  (PhVector magnitude:10 direction: 0) components.! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:24'!
testComponentsN
	self assert: 0@10 closeTo:  (PhVector magnitude:10 direction: 90) components! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:24'!
testComponentsNbE
	self assert: 3@4 closeTo:  (PhVector magnitude:5 direction: 53.130102354156) components! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:24'!
testComponentsNbW
	self assert: -3@4 closeTo:  (PhVector magnitude:5 direction: 126.869897645844) components! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:25'!
testComponentsS
	self assert: 0@-10 closeTo:  (PhVector magnitude:10 direction: 270) components! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:25'!
testComponentsSbE
	self assert: 3@-4 closeTo:  (PhVector magnitude:5 direction: 306.869897645844) components! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:25'!
testComponentsSbW
	self assert: -3@-4 closeTo:  (PhVector magnitude:5 direction: 233.130102354156) components! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:25'!
testComponentsW
	self assert: -10@0 closeTo:  (PhVector magnitude:10 direction: 180) components! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/13/2008 16:34'!
testEquals
	self 
		assert: (PhVector magnitude: 2  direction: 45)
		equals: (PhVector magnitude: 2 direction: 45)! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 19:27'!
testFromComponentsE
	self assert: (PhVector magnitude: 10 direction: 0)  equals:  (PhVector fromComponents: 10@0)! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 19:26'!
testFromComponentsN
	self assert: (PhVector magnitude: 10 direction: 90)  equals:  (PhVector fromComponents: 0@10)! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:06'!
testFromComponentsNbE
	self assert: (PhVector magnitude:5 direction: 53.130102354156)  equals:  (PhVector fromComponents: 3@4)! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:08'!
testFromComponentsNbW
	self assert: (PhVector magnitude:5 direction: 126.869897645844)  equals:  (PhVector fromComponents: -3@4)! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 19:26'!
testFromComponentsS
	self assert: (PhVector magnitude: 10 direction: 270)  equals:  (PhVector fromComponents: 0@-10)! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:11'!
testFromComponentsSbE
	self assert: (PhVector magnitude:5 direction: 306.869897645844)  equals:  (PhVector fromComponents: 3@-4)! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 20:10'!
testFromComponentsSbW
	self assert: (PhVector magnitude:5 direction: 233.130102354156)  equals:  (PhVector fromComponents: -3@-4)! !

!PhVectorTest methodsFor: 'as yet unclassified' stamp: 'srh 12/14/2008 19:27'!
testFromComponentsW
	self assert: (PhVector magnitude: 10 direction: 180)  equals:  (PhVector fromComponents: -10@0)! !
